#include <PSX/Auxiliary/big_endian.hpp>
#include <PSX/File/file_processor_helper.hpp>
#include <PSX/SPU/spu.hpp>
#include <stdio.hpp>

#ifdef __SUPPORT_VAG__
namespace JabyEngine {
    namespace FileProcessor {
        struct VAGHeader {
            char     id[4];
            uint32_t version;
            uint32_t reserved;
            uint32_t sample_size;
            uint32_t sample_frequency;
            uint8_t  reserved_2[12];
            char     name[16];

            constexpr uint32_t get_version() const {
                return read_be(this->version);
            }

            constexpr uint32_t get_sample_size() const {
                return read_be(this->sample_size);
            }

            constexpr uint32_t get_sample_frequency() const {
                return read_be(this->sample_frequency);
            }
        };

        struct VAGState {
            uint32_t      voice_id;
            size_t        bytes_left;
            SPU::SRAM_Adr adr;

            static constexpr VAGState create(uint32_t voice_id) {
                return VAGState{.voice_id = voice_id, .bytes_left = 0, .adr = 0};
            }
        };

        static Progress parse_header(State::Configuration& config, VAGState& state) {
            if(config.data_bytes >= sizeof(VAGHeader)) {
                const auto& header = *reinterpret_cast<const VAGHeader*>(config.data_adr);

                state.bytes_left = header.get_sample_size();
                state.adr        = SPU::voice[state.voice_id].allocate(SPU_IO::SampleRate::from_HZ(header.get_sample_frequency()), state.bytes_left);                

                // TODO: Parse samples
                return Progress::Error;
            }

            return Progress::InProgress;
        }

        State create(const uint32_t* data_adr, const VAG& file) {
            return State::from(VAGState::create(file.voice_number), data_adr, parse_header);
        }
    }
}
#endif // __SUPPORT_VAG__